# $NetBSD: t_bridge.sh,v 1.1 2020/06/25 18:30:42 jruoho Exp $
#
# Copyright (c) 2020 The NetBSD Foundation, Inc.
# All rights reserved.
#
# This code is derived from software contributed to The NetBSD Foundation
# by Jukka Ruohonen.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
bridges="/tmp/bridges"

atf_test_case manybridges cleanup
manybridges_head() {
	atf_set "require.user" "root"
	atf_set "descr" "Test creating many, many bridge(4)'s"
}

manybridges_body() {

	seq 65535 65000 > $bridges # Try to avoid stalling automated runs.

	while read bridge; do

		ifconfig "bridge$bridge" >/dev/null 2>&1

		if [ $? -eq 0 ]; then
			echo "Skipping existing bridge$bridge"
			continue
		fi

		ifconfig "bridge$bridge" create
		echo "Created bridge$bridge"

	done < $bridges
}

manybridges_cleanup() {

	if [ -f $bridges ]; then

		while read bridge; do
			ifconfig "bridge$bridge" destroy >/dev/null 2>&1
			echo "Burnt down bridge$bridge"
		done < $bridges

		rm $bridges
	fi
}

atf_init_test_cases() {
	atf_add_test_case manybridges
}
